@@ -6,8 +6,11 @@ import pathlib
 import re
 import textwrap
 from abc import abstractmethod
+from inspect import cleandoc
 from typing import Union
+
 import yaml
+from docutils.core import publish_parts
 
 try:
     import tomli as toml
@@ -16,13 +19,13 @@ try:
 except ImportError:
     HAS_TOML = False
 
-from traitlets import default, List
+from traitlets import List, default
 from traitlets.config import Application, Config, Configurable
 
 from .. import __version__ as version
 from . import Provenance
 from .component import Component
-from .logging import DEFAULT_LOGGING, ColoredFormatter, create_logging_config
+from .logging import ColoredFormatter, create_logging_config
 from .traits import Bool, Dict, Enum, Path
 
 __all__ = ["Tool", "ToolConfigurationError"]
@@ -131,8 +134,6 @@ class Tool(Application):
         trait=Path(
             exists=True,
             directory_ok=False,
-            allow_none=True,
-            default_value=None,
             help=(
                 "List of configuration files with parameters to load "
                 "in addition to command-line parameters. "
@@ -144,7 +145,7 @@ class Tool(Application):
         )
     ).tag(config=True)
 
-    log_config = Dict(default_value=DEFAULT_LOGGING).tag(config=True)
+    log_config = Dict(default_value={}).tag(config=True)
     log_file = Path(
         default_value=None,
         exists=None,
@@ -195,8 +196,8 @@ class Tool(Application):
 
         # tools defined in other modules should have those modules as base
         # logging name
-        module_name = self.__class__.__module__.split(".")[0]
-        self.log = logging.getLogger(f"{module_name}.{self.name}")
+        self.module_name = self.__class__.__module__.split(".")[0]
+        self.log = logging.getLogger(f"{self.module_name}.{self.name}")
         self.trait_warning_handler = CollectTraitWarningsHandler()
         self.update_logging_config()
 
@@ -231,7 +232,6 @@ class Tool(Application):
         path: Union[str, pathlib.Path]
             config file to load. [yaml, toml, json, py] formats are supported
         """
-
         path = pathlib.Path(path)
 
         if path.suffix in [".yaml", ".yml"]:
@@ -257,6 +257,7 @@ class Tool(Application):
             log_file_level=self.log_file_level,
             log_config=self.log_config,
             quiet=self.quiet,
+            module=self.module_name,
         )
 
         logging.config.dictConfig(cfg)
@@ -312,7 +313,7 @@ class Tool(Application):
         after `Tool.start` when `Tool.run` is called."""
         self.log.info("Goodbye")
 
-    def run(self, argv=None):
+    def run(self, argv=None, raises=False):
         """Run the tool. This automatically calls `initialize()`,
         `start()` and `finish()`
 
@@ -360,6 +361,8 @@ class Tool(Application):
             self.log.exception(f"Caught unexpected exception: {err}")
             Provenance().finish_activity(activity_name=self.name, status="error")
             exit_status = 1  # any other error
+            if raises:
+                raise err
         finally:
             if not {"-h", "--help", "--help-all"}.intersection(self.argv):
                 self.write_provenance()
@@ -401,30 +404,47 @@ class Tool(Application):
         """nice HTML rep, with blue for non-default values"""
         traits = self.traits()
         name = self.__class__.__name__
+        docstring = (
+            publish_parts(
+                cleandoc(self.__class__.__doc__ or self.description), writer_name="html"
+            )["html_body"]
+            or "Undocumented"
+        )
         lines = [
             f"<b>{name}</b>",
-            f"<p> {self.__class__.__doc__ or self.description} </p>",
+            f"<p> {docstring} </p>",
             "<table>",
+            "    <colgroup>",
+            "        <col span='1' style=' '>",
+            "        <col span='1' style='width: 20em;'>",
+            "        <col span='1' >",
+            "    </colgroup>",
+            "    <tbody>",
         ]
         for key, val in self.get_current_config()[name].items():
-            # after running setup, also the subcomponents are in the current config
-            # which are not in traits
-            if key not in traits:
-                continue
-
-            default = traits[key].default_value
-            thehelp = f"{traits[key].help} (default: {default})"
-            lines.append(f"<tr><th>{key}</th>")
-            if val != default:
-                lines.append(f"<td><span style='color:blue'>{val}</span></td>")
-            else:
-                lines.append(f"<td>{val}</td>")
-            lines.append(f'<td style="text-align:left"><i>{thehelp}</i></td></tr>')
+            htmlval = (
+                str(val).replace("/", "/<wbr>").replace("_", "_<wbr>")
+            )  # allow breaking at boundary
 
+            # traits of the current component
+            if key in traits:
+                thehelp = f"{traits[key].help} (default: {traits[key].default_value})"
+                lines.append(f"<tr><th>{key}</th>")
+                if val != traits[key].default_value:
+                    lines.append(
+                        f"<td style='text-align: left;'><span style='color:blue; max-width:30em;'>{htmlval}</span></td>"
+                    )
+                else:
+                    lines.append(f"<td style='text-align: left;'>{htmlval}</td>")
+                lines.append(
+                    f"<td style='text-align: left;'><i>{thehelp}</i></td></tr>"
+                )
+        lines.append("    </tbody>")
         lines.append("</table>")
-        lines.append("<p><i>Components:</i>")
+        lines.append("<p><b>Components:</b>")
         lines.append(", ".join([x.__name__ for x in self.classes]))
         lines.append("</p>")
+        lines.append("</div>")
 
         return "\n".join(lines)
 
@@ -508,7 +528,7 @@ def export_tool_config_to_commented_yaml(tool_instance: Tool, classes=None):
     return "\n".join(lines)
 
 
-def run_tool(tool: Tool, argv=None, cwd=None):
+def run_tool(tool: Tool, argv=None, cwd=None, raises=False):
     """
     Utility run a certain tool in a python session without exitinig
 
@@ -522,7 +542,7 @@ def run_tool(tool: Tool, argv=None, cwd=None):
     try:
         # switch to cwd for running and back after
         os.chdir(cwd)
-        tool.run(argv or [])
+        tool.run(argv or [], raises=raises)
     except SystemExit as e:
         return e.code
     finally:
