@@ -2,18 +2,20 @@
 Algorithms for the data volume reduction.
 """
 from abc import abstractmethod
+
 import numpy as np
+
 from ctapipe.containers import DL1CameraContainer
-from ctapipe.image import TailcutsImageCleaner
 from ctapipe.core import TelescopeComponent
 from ctapipe.core.traits import (
-    IntTelescopeParameter,
     BoolTelescopeParameter,
+    IntTelescopeParameter,
     TelescopeParameter,
     create_class_enum_trait,
 )
-from ctapipe.image.extractor import ImageExtractor
+from ctapipe.image import TailcutsImageCleaner
 from ctapipe.image.cleaning import dilate
+from ctapipe.image.extractor import ImageExtractor
 
 __all__ = ["DataVolumeReducer", "NullDataVolumeReducer", "TailCutsDataVolumeReducer"]
 
@@ -38,7 +40,7 @@ class DataVolumeReducer(TelescopeComponent):
         self.subarray = subarray
         super().__init__(config=config, parent=parent, subarray=subarray, **kwargs)
 
-    def __call__(self, waveforms, telid=None, selected_gain_channel=None):
+    def __call__(self, waveforms, tel_id=None, selected_gain_channel=None):
         """
         Call the relevant functions to perform data volume reduction on the
         waveforms.
@@ -48,7 +50,7 @@ class DataVolumeReducer(TelescopeComponent):
         waveforms: ndarray
             Waveforms stored in a numpy array of shape
             (n_pix, n_samples).
-        telid: int
+        tel_id: int
             The telescope id. Required for the 'image_extractor' and
             'camera.geometry' in 'TailCutsDataVolumeReducer'.
         selected_gain_channel: ndarray
@@ -62,12 +64,12 @@ class DataVolumeReducer(TelescopeComponent):
             Mask of selected pixels.
         """
         mask = self.select_pixels(
-            waveforms, telid=telid, selected_gain_channel=selected_gain_channel
+            waveforms, tel_id=tel_id, selected_gain_channel=selected_gain_channel
         )
         return mask
 
     @abstractmethod
-    def select_pixels(self, waveforms, telid=None, selected_gain_channel=None):
+    def select_pixels(self, waveforms, tel_id=None, selected_gain_channel=None):
         """
         Abstract method to be defined by a DataVolumeReducer subclass.
         Call the relevant functions for the required pixel selection.
@@ -77,7 +79,7 @@ class DataVolumeReducer(TelescopeComponent):
         waveforms: ndarray
             Waveforms stored in a numpy array of shape
             (n_pix, n_samples).
-        telid: int
+        tel_id: int
             The telescope id. Required for the 'image_extractor' and
             'camera.geometry' in 'TailCutsDataVolumeReducer'.
         selected_gain_channel: ndarray
@@ -96,7 +98,7 @@ class NullDataVolumeReducer(DataVolumeReducer):
     Perform no data volume reduction
     """
 
-    def select_pixels(self, waveforms, telid=None, selected_gain_channel=None):
+    def select_pixels(self, waveforms, tel_id=None, selected_gain_channel=None):
         mask = waveforms != 0
         return mask
 
@@ -177,18 +179,23 @@ class TailCutsDataVolumeReducer(DataVolumeReducer):
             self.image_extractor_type = [("type", "*", name)]
             self.image_extractors[name] = image_extractor
 
-    def select_pixels(self, waveforms, telid=None, selected_gain_channel=None):
-        camera_geom = self.subarray.tel[telid].camera.geometry
+    def select_pixels(self, waveforms, tel_id=None, selected_gain_channel=None):
+        camera_geom = self.subarray.tel[tel_id].camera.geometry
         # Pulse-integrate waveforms
-        extractor = self.image_extractors[self.image_extractor_type.tel[telid]]
+        extractor = self.image_extractors[self.image_extractor_type.tel[tel_id]]
+        # do not treat broken pixels in data volume reduction
+        broken_pixels = np.zeros(camera_geom.n_pixels, dtype=bool)
         dl1: DL1CameraContainer = extractor(
-            waveforms, telid=telid, selected_gain_channel=selected_gain_channel
+            waveforms,
+            tel_id=tel_id,
+            selected_gain_channel=selected_gain_channel,
+            broken_pixels=broken_pixels,
         )
 
         # 1) Step: TailcutCleaning at first
-        mask = self.cleaner(telid, dl1.image)
+        mask = self.cleaner(tel_id, dl1.image)
         pixels_above_boundary_thresh = (
-            dl1.image >= self.cleaner.boundary_threshold_pe.tel[telid]
+            dl1.image >= self.cleaner.boundary_threshold_pe.tel[tel_id]
         )
         mask_in_loop = np.array([])
         # 2) Step: Add iteratively all pixels with Signal
@@ -196,13 +203,13 @@ class TailCutsDataVolumeReducer(DataVolumeReducer):
         #          'dilate' until no new pixels were added.
         while (
             not np.array_equal(mask, mask_in_loop)
-            and self.do_boundary_dilation.tel[telid]
+            and self.do_boundary_dilation.tel[tel_id]
         ):
             mask_in_loop = mask
             mask = dilate(camera_geom, mask) & pixels_above_boundary_thresh
 
         # 3) Step: Adding Pixels with 'dilate' to get more conservative.
-        for _ in range(self.n_end_dilates.tel[telid]):
+        for _ in range(self.n_end_dilates.tel[tel_id]):
             mask = dilate(camera_geom, mask)
 
         return mask
