@@ -1,19 +1,22 @@
-from .camera import CameraDescription, CameraGeometry, CameraReadout, PixelShape
 from .atmosphere import get_atmosphere_profile_functions
-from .telescope import TelescopeDescription
-from .optics import OpticsDescription
-from .subarray import SubarrayDescription
+from .camera import CameraDescription, CameraGeometry, CameraReadout, PixelShape
 from .guess import guess_telescope
-
+from .optics import FocalLengthKind, OpticsDescription, ReflectorShape, SizeType
+from .subarray import SubarrayDescription, UnknownTelescopeID
+from .telescope import TelescopeDescription
 
 __all__ = [
     "CameraDescription",
     "CameraGeometry",
     "CameraReadout",
     "get_atmosphere_profile_functions",
-    "TelescopeDescription",
-    "OpticsDescription",
-    "SubarrayDescription",
     "guess_telescope",
+    "OpticsDescription",
     "PixelShape",
+    "SubarrayDescription",
+    "TelescopeDescription",
+    "UnknownTelescopeID",
+    "FocalLengthKind",
+    "ReflectorShape",
+    "SizeType",
 ]
