@@ -2,15 +2,16 @@ from .eventseeker import EventSeeker
 from .eventsource import EventSource
 from .hdf5tableio import HDF5TableReader, HDF5TableWriter
 from .tableio import TableWriter, TableReader
+from .tableloader import TableLoader
 from .datalevels import DataLevel
-from .astropy_helpers import read_table
-from .dl1writer import DL1Writer
+from .astropy_helpers import read_table, write_table
+from .datawriter import DataWriter, DATA_MODEL_VERSION
 
 from ..core.plugins import detect_and_import_io_plugins
 
 # import event sources to make them visible to EventSource.from_url
 from .simteleventsource import SimTelEventSource
-from .dl1eventsource import DL1EventSource
+from .hdf5eventsource import HDF5EventSource, get_hdf5_datalevels
 
 # import IO plugins with their event sources
 detect_and_import_io_plugins()
@@ -21,11 +22,15 @@ __all__ = [
     "HDF5TableReader",
     "TableWriter",
     "TableReader",
+    "TableLoader",
     "EventSeeker",
     "EventSource",
     "SimTelEventSource",
-    "DL1EventSource",
+    "HDF5EventSource",
     "DataLevel",
     "read_table",
-    "DL1Writer",
+    "write_table",
+    "DataWriter",
+    "DATA_MODEL_VERSION",
+    "get_hdf5_datalevels",
 ]
