@@ -1,17 +1,23 @@
 """
 Handles reading of different event/waveform containing files
 """
+import warnings
 from abc import abstractmethod
-from traitlets.config.loader import LazyConfigValue
-from typing import Tuple, List, Generator
+from typing import Dict, Generator, List, Tuple
 
+from traitlets.config.loader import LazyConfigValue
+
+from ..containers import (
+    ArrayEventContainer,
+    ObservationBlockContainer,
+    SchedulingBlockContainer,
+    SimulationConfigContainer,
+)
+from ..core import Provenance, ToolConfigurationError
+from ..core.component import Component, find_config_in_hierarchy, non_abstract_children
+from ..core.traits import CInt, Int, Path, Set, TraitError, Undefined
 from ..instrument import SubarrayDescription
 from .datalevels import DataLevel
-from ..containers import ArrayEventContainer
-from ..core import ToolConfigurationError, Provenance
-from ..core.component import Component, non_abstract_children, find_config_in_hierarchy
-from ..core.traits import Path, Int, CInt, Set, Undefined
-
 
 __all__ = ["EventSource"]
 
@@ -83,11 +89,7 @@ class EventSource(Component):
         generated events. If None, all available telescopes are used.
     """
 
-    input_url = Path(
-        directory_ok=False,
-        exists=True,
-        help="Path to the input file containing events.",
-    ).tag(config=True)
+    input_url = Path(help="Path to the input file containing events.").tag(config=True)
 
     max_events = Int(
         None,
@@ -208,11 +210,38 @@ class EventSource(Component):
 
         """
 
+    @property
+    def simulation_config(self) -> Dict[int, SimulationConfigContainer]:
+        """The simulation configurations of all observations provided by the
+        EventSource, or None if the source does not provide simulated data
+
+        Returns
+        -------
+        Dict[int,ctapipe.containers.SimulationConfigContainer] | None
+        """
+        return None
+
+    @property
+    @abstractmethod
+    def observation_blocks(self) -> Dict[int, ObservationBlockContainer]:
+        """
+        Obtain the ObservationConfigurations from the EventSource, indexed by obs_id
+        """
+        pass
+
+    @property
+    @abstractmethod
+    def scheduling_blocks(self) -> Dict[int, SchedulingBlockContainer]:
+        """
+        Obtain the ObservationConfigurations from the EventSource, indexed by obs_id
+        """
+        pass
+
     @property
     @abstractmethod
     def is_simulation(self) -> bool:
         """
-        Weither the currently opened file is simulated
+        Whether the currently opened file is simulated
 
         Returns
         -------
@@ -243,7 +272,6 @@ class EventSource(Component):
         return any(dl in self.datalevels for dl in datalevels)
 
     @property
-    @abstractmethod
     def obs_ids(self) -> List[int]:
         """
         The observation ids of the runs located in the file
@@ -253,6 +281,7 @@ class EventSource(Component):
         -------
         list[int]
         """
+        return list(self.observation_blocks.keys())
 
     @abstractmethod
     def _generator(self) -> Generator[ArrayEventContainer, None, None]:
@@ -298,8 +327,18 @@ class EventSource(Component):
         available_classes = non_abstract_children(cls)
 
         for subcls in available_classes:
-            if subcls.is_compatible(input_url):
-                return subcls
+            try:
+                if subcls.is_compatible(input_url):
+                    return subcls
+            except Exception as e:
+                warnings.warn(f"{subcls.__name__}.is_compatible raised exception: {e}")
+
+        # provide a more helpful error for non-existing input_url
+        if not input_url.exists():
+            raise TraitError(
+                f"input_url {input_url} is not an existing file "
+                " and no EventSource implementation claimed compatibility"
+            )
 
         raise ValueError(
             "Cannot find compatible EventSource for \n"
