@@ -22,19 +22,19 @@ them (as in `Activity.from_provenance()`)
     some_astropy_table.write("output.ecsv")
 
 """
+import os
+import pwd
 import uuid
 import warnings
 from collections import OrderedDict
-import os
-import pwd
+from contextlib import ExitStack
+from pathlib import Path
 
 import tables
 from astropy.time import Time
 from tables import NaturalNameWarning
-from traitlets import Enum, Instance, List, Unicode, default, HasTraits
+from traitlets import Enum, HasTraits, Instance, List, Unicode, UseEnum, default
 from traitlets.config import Configurable
-from contextlib import ExitStack
-from pathlib import Path
 
 from ..core.traits import AstroTime
 from .datalevels import DataLevel
@@ -51,11 +51,22 @@ __all__ = [
 ]
 
 
-CONVERSIONS = {Time: lambda t: t.utc.iso, list: str}
+CONVERSIONS = {
+    Time: lambda t: t.utc.iso,
+    list: lambda l: ",".join([convert(elem) for elem in l]),
+    DataLevel: lambda d: d.name,
+}
+
+
+def convert(value):
+    """Convert to representation suitable for header infos, such as hdf5 or fits"""
+    if (conv := CONVERSIONS.get(type(value))) is not None:
+        return conv(value)
+    return value
 
 
 class Contact(Configurable):
-    """ Contact information """
+    """Contact information"""
 
     name = Unicode("unknown").tag(config=True)
     email = Unicode("unknown").tag(config=True)
@@ -63,7 +74,7 @@ class Contact(Configurable):
 
     @default("name")
     def default_name(self):
-        """ if no name specified, use the system's user name"""
+        """if no name specified, use the system's user name"""
         try:
             return pwd.getpwuid(os.getuid()).pw_gecos
         except RuntimeError:
@@ -80,7 +91,7 @@ class Product(HasTraits):
     creation_time = AstroTime()
     id_ = Unicode(help="leave unspecified to automatically generate a UUID")
     data_category = Enum(["Sim", "A", "B", "C", "Other"], "Other")
-    data_level = List(Enum([level.name for level in DataLevel]))
+    data_levels = List(UseEnum(DataLevel))
     data_association = Enum(["Subarray", "Telescope", "Target", "Other"], "Other")
     data_model_name = Unicode("unknown")
     data_model_version = Unicode("unknown")
@@ -90,17 +101,17 @@ class Product(HasTraits):
     # pylint: disable=no-self-use
     @default("creation_time")
     def default_time(self):
-        """ return current time by default """
+        """return current time by default"""
         return Time.now().iso
 
     @default("id_")
     def default_product_id(self):
-        """ default id is a UUID """
+        """default id is a UUID"""
         return str(uuid.uuid4())
 
 
 class Process(HasTraits):
-    """ Process (top-level workflow) information """
+    """Process (top-level workflow) information"""
 
     type_ = Enum(["Observation", "Simulation", "Other"], "Other")
     subtype = Unicode("")
@@ -108,11 +119,11 @@ class Process(HasTraits):
 
 
 class Activity(HasTraits):
-    """ Activity (tool) information """
+    """Activity (tool) information"""
 
     @classmethod
     def from_provenance(cls, activity):
-        """ construct Activity metadata from existing ActivityProvenance object"""
+        """construct Activity metadata from existing ActivityProvenance object"""
         return Activity(
             name=activity["activity_name"],
             type_="software",
@@ -132,12 +143,12 @@ class Activity(HasTraits):
     # pylint: disable=no-self-use
     @default("start_time")
     def default_time(self):
-        """ default time is now """
+        """default time is now"""
         return Time.now().iso
 
 
-class Instrument(HasTraits):
-    """ Instrumental Context """
+class Instrument(Configurable):
+    """Instrumental Context"""
 
     site = Enum(
         [
@@ -155,7 +166,7 @@ class Instrument(HasTraits):
         "Other",
         help="Which site of CTA (or external telescope) "
         "this instrument is associated with",
-    )
+    ).tag(config=True)
     class_ = Enum(
         [
             "Array",
@@ -170,15 +181,21 @@ class Instrument(HasTraits):
             "Other",
         ],
         "Other",
-    )
-    type_ = Unicode("unspecified")
-    subtype = Unicode("unspecified")
-    version = Unicode("unspecified")
-    id_ = Unicode("unspecified")
+    ).tag(config=True)
+    type_ = Unicode("unspecified").tag(config=True)
+    subtype = Unicode("unspecified").tag(config=True)
+    version = Unicode("unspecified").tag(config=True)
+    id_ = Unicode("unspecified").tag(config=True)
+
+    def __repr__(self):
+        return (
+            f"Contact({self.site=}, {self.class_=}, {self.type_=}, "
+            f"{self.subtype=}, {self.version=}, {self.id_=})"
+        )
 
 
 def _to_dict(hastraits_instance, prefix=""):
-    """ helper to convert a HasTraits to a dict with keys
+    """helper to convert a HasTraits to a dict with keys
     in the required CTA format (upper-case, space separated)
     """
     res = {}
@@ -192,15 +209,15 @@ def _to_dict(hastraits_instance, prefix=""):
         val = trait.get(hastraits_instance)
 
         # apply type conversions
-        val = CONVERSIONS.get(type(val), lambda v: v)(val)
+        val = convert(val)
         res[key] = val
 
     return res
 
 
 class Reference(HasTraits):
-    """ All the reference Metadata required for a CTA output file, plus a way to turn
-    it into a dict() for easy addition to the header of a file """
+    """All the reference Metadata required for a CTA output file, plus a way to turn
+    it into a dict() for easy addition to the header of a file"""
 
     contact = Instance(Contact)
     product = Instance(Product)
@@ -225,7 +242,7 @@ class Reference(HasTraits):
         return meta
 
 
-def write_to_hdf5(metadata, h5file, path='/'):
+def write_to_hdf5(metadata, h5file, path="/"):
     """
     Write metadata fields to a PyTables HDF5 file handle.
 
@@ -246,7 +263,7 @@ def write_to_hdf5(metadata, h5file, path='/'):
             node._v_attrs[key] = value  # pylint: disable=protected-access
 
 
-def read_metadata(h5file, path='/'):
+def read_metadata(h5file, path="/"):
     """
     Read metadata from an hdf5 file
 
@@ -274,4 +291,4 @@ def read_metadata(h5file, path='/'):
 
         node = h5file.get_node(path)
         metadata = {key: node._v_attrs[key] for key in node._v_attrs._f_list()}
-    return metadata
+        return metadata
