@@ -1,5 +1,6 @@
 import astropy.units as u
 import numpy as np
+
 from ctapipe.io import DataLevel, EventSource, HDF5EventSource
 from ctapipe.utils import get_dataset_path
 
@@ -19,8 +20,8 @@ def test_metadata(dl1_file):
             DataLevel.DL1_IMAGES,
             DataLevel.DL1_PARAMETERS,
         }
-        assert list(source.obs_ids) == [2]
-        assert source.simulation_config[2].corsika_version == 7710
+        assert list(source.obs_ids) == [1]
+        assert source.simulation_config[1].corsika_version == 7710
 
 
 def test_subarray(dl1_file):
@@ -78,6 +79,7 @@ def test_simulation_info(dl1_file):
 
 def test_dl1_a_only_data(dl1_image_file):
     with HDF5EventSource(input_url=dl1_image_file) as source:
+        assert source.datalevels == (DataLevel.DL1_IMAGES,)
         for event in source:
             for tel in event.dl1.tel:
                 assert event.dl1.tel[tel].image.any()
@@ -87,6 +89,7 @@ def test_dl1_b_only_data(dl1_parameters_file):
     reco_lons = []
     reco_concentrations = []
     with HDF5EventSource(input_url=dl1_parameters_file) as source:
+        assert source.datalevels == (DataLevel.DL1_PARAMETERS,)
         for event in source:
             for tel in event.dl1.tel:
                 reco_lons.append(
@@ -132,8 +135,51 @@ def test_read_r1(r1_hdf5_file):
     with HDF5EventSource(input_url=r1_hdf5_file) as source:
         e = None
 
+        assert source.datalevels == (DataLevel.R1,)
+
         for e in source:
             pass
 
         assert e is not None
         assert e.count == 3
+
+
+def test_trigger_allowed_tels(dl1_proton_file):
+    with HDF5EventSource(
+        input_url=dl1_proton_file, allowed_tels={1, 2, 3, 4, 5, 10}
+    ) as s:
+        print()
+        i = 0
+        for i, e in enumerate(s):
+            assert e.count == i
+            assert set(e.trigger.tels_with_trigger) == e.trigger.tel.keys()
+            assert len(e.trigger.tels_with_trigger) > 1
+
+        assert i == 1
+
+
+def test_read_dl2(dl2_shower_geometry_file):
+    algorithm = "HillasReconstructor"
+
+    with HDF5EventSource(dl2_shower_geometry_file) as s:
+        assert s.datalevels == (
+            DataLevel.DL1_IMAGES,
+            DataLevel.DL1_PARAMETERS,
+            DataLevel.DL2,
+        )
+
+        e = next(iter(s))
+        assert algorithm in e.dl2.stereo.geometry
+        assert e.dl2.stereo.geometry[algorithm].alt is not None
+        assert e.dl2.stereo.geometry[algorithm].az is not None
+        assert e.dl2.stereo.geometry[algorithm].telescopes is not None
+        assert e.dl2.stereo.geometry[algorithm].prefix == algorithm
+
+        tel_mask = e.dl2.stereo.geometry[algorithm].telescopes
+        tel_ids = s.subarray.tel_mask_to_tel_ids(tel_mask)
+        for tel_id in tel_ids:
+            assert tel_id in e.dl2.tel
+            assert algorithm in e.dl2.tel[tel_id].impact
+            impact = e.dl2.tel[tel_id].impact[algorithm]
+            assert impact.prefix == algorithm + "_tel"
+            assert impact.distance is not None
