@@ -2,12 +2,13 @@
 Calibrate dl0 data to dl1, and plot the photoelectron images.
 """
 from copy import copy
+
 from matplotlib import pyplot as plt
 from matplotlib.backends.backend_pdf import PdfPages
 
 from ..calib import CameraCalibrator
 from ..core import Component, Tool
-from ..core.traits import Bool, Path, flag, Int, classes_with_traits
+from ..core.traits import Bool, Int, Path, classes_with_traits, flag
 from ..image.extractor import ImageExtractor
 from ..io import EventSource
 from ..io.datalevels import DataLevel
@@ -67,18 +68,18 @@ class ImagePlotter(Component):
             self.log.info(f"Creating PDF: {self.output_path}")
             self.pdf = PdfPages(self.output_path)
 
-    def plot(self, event, telid):
-        image = event.dl1.tel[telid].image
-        peak_time = event.dl1.tel[telid].peak_time
+    def plot(self, event, tel_id):
+        image = event.dl1.tel[tel_id].image
+        peak_time = event.dl1.tel[tel_id].peak_time
 
-        if self._current_tel != telid:
-            self._current_tel = telid
+        if self._current_tel != tel_id:
+            self._current_tel = tel_id
 
             self.ax_intensity.cla()
             self.ax_peak_time.cla()
 
             # Redraw camera
-            geom = self.subarray.tel[telid].camera.geometry
+            geom = self.subarray.tel[tel_id].camera.geometry
             self.c_intensity = CameraDisplay(geom, ax=self.ax_intensity)
 
             time_cmap = copy(plt.get_cmap("RdBu_r"))
@@ -112,7 +113,7 @@ class ImagePlotter(Component):
 
         self.fig.suptitle(
             "Event_index={}  Event_id={}  Telescope={}".format(
-                event.count, event.index.event_id, telid
+                event.count, event.index.event_id, tel_id
             )
         )
 
@@ -187,8 +188,8 @@ class DisplayDL1Calib(Tool):
                     continue
                 tel_list = [self.telescope]
 
-            for telid in tel_list:
-                self.plotter.plot(event, telid)
+            for tel_id in tel_list:
+                self.plotter.plot(event, tel_id)
 
     def finish(self):
         self.plotter.finish()
