@@ -1,27 +1,28 @@
 """
 Merge DL1-files from ctapipe-process tool
 """
-import sys
 import os
+import sys
 from argparse import ArgumentParser
 from pathlib import Path
-from traitlets import List
 
 import tables
-import numpy as np
 from tqdm.auto import tqdm
+from traitlets import List
+
+from ctapipe.utils.arrays import recarray_drop_columns
 
-from ..io import metadata as meta, HDF5EventSource, get_hdf5_datalevels
-from ..io import HDF5TableWriter
 from ..core import Provenance, Tool, traits
-from ..core.traits import Bool, Set, Unicode, flag, CInt
+from ..core.traits import Bool, CInt, Set, Unicode, flag
 from ..instrument import SubarrayDescription
-
+from ..io import HDF5EventSource, HDF5TableWriter, get_hdf5_datalevels
+from ..io import metadata as meta
 
 PROV = Provenance()
 
 VERSION_KEY = "CTA PRODUCT DATA MODEL VERSION"
 IMAGE_STATISTICS_PATH = "/dl1/service/image_statistics"
+DL2_STATISTICS_GROUP = "/dl2/service/tel_event_statistics"
 
 required_nodes = {
     "/dl1/event/subarray/trigger",
@@ -35,11 +36,14 @@ optional_nodes = {
     "/simulation/event/telescope/parameters",
     "/dl1/event/telescope/parameters",
     "/dl1/event/telescope/images",
-    "/dl1/service/image_statistics",
-    "/dl1/service/image_statistics.__table_column_meta__",
     "/dl2/event/subarray/geometry",
 }
 
+observation_configuration_nodes = {
+    "/configuration/observation/observation_block",
+    "/configuration/observation/scheduling_block",
+}
+
 simulation_nodes = {
     "/simulation/event/subarray/shower",
     "/simulation/event/telescope/parameters",
@@ -47,10 +51,6 @@ simulation_nodes = {
     "/simulation/service/shower_distribution",
     "/configuration/simulation/run",
 }
-service_nodes = {
-    "/dl1/service/image_statistics",
-    "/dl1/service/image_statistics.__table_column_meta__",
-}
 nodes_with_tels = {
     "/dl1/monitoring/telescope/pointing",
     "/dl1/event/telescope/parameters",
@@ -58,13 +58,16 @@ nodes_with_tels = {
     "/simulation/event/telescope/parameters",
     "/simulation/event/telescope/images",
 }
-image_nodes = {"/simulation/event/telescope/images", "/dl1/event/telescope/images"}
+image_nodes = {
+    "/dl1/event/telescope/images",
+}
 parameter_nodes = {
     "/simulation/event/telescope/parameters",
     "/dl1/event/telescope/parameters",
 }
 
-simulation_images = {"/simulation/event/telescope/images"}
+SIMULATED_IMAGE_GROUP = "/simulation/event/telescope/images"
+simulation_images = {SIMULATED_IMAGE_GROUP}
 
 dl2_subarray_nodes = {"/dl2/event/subarray/geometry"}
 
@@ -73,12 +76,12 @@ all_nodes = (
     required_nodes
     | optional_nodes
     | simulation_nodes
-    | service_nodes
     | nodes_with_tels
     | image_nodes
     | parameter_nodes
     | simulation_images
     | dl2_subarray_nodes
+    | observation_configuration_nodes
 )
 
 
@@ -235,13 +238,13 @@ class MergeTool(Tool):
         # setup required nodes
         self.usable_nodes = all_nodes.copy()
 
-        if self.skip_simu_images is True:
+        if self.skip_simu_images:
             self.usable_nodes -= simulation_images
 
-        if self.skip_images is True:
+        if self.skip_images:
             self.usable_nodes -= image_nodes
 
-        if self.skip_parameters is True:
+        if self.skip_parameters:
             self.usable_nodes -= parameter_nodes
 
         # Use first file as reference to setup nodes to merge
@@ -318,33 +321,35 @@ class MergeTool(Tool):
 
         return broken
 
-    def add_image_statistics(self, file):
-        # Creates table for image statistics and adds the entries together.
-        # This does not append rows to the existing table
+    def add_statistics(self, file):
+        if IMAGE_STATISTICS_PATH in file.root:
+            self.add_statistics_node(file, IMAGE_STATISTICS_PATH)
 
-        if IMAGE_STATISTICS_PATH in self.output_file:
-            table_out = self.output_file.root[IMAGE_STATISTICS_PATH]
-            table_in = file.root[IMAGE_STATISTICS_PATH]
+        if DL2_STATISTICS_GROUP in file.root:
+            for node in file.root[DL2_STATISTICS_GROUP]._v_children.values():
+                self.add_statistics_node(file, node._v_pathname)
 
-            for row in range(len(table_in)):
-                table_out.cols.counts[row] = np.add(
-                    table_out.cols.counts[row], table_in.cols.counts[row]
-                )
-                table_out.cols.cumulative_counts[row] = np.add(
-                    table_out.cols.cumulative_counts[row],
-                    table_in.cols.cumulative_counts[row],
+    def add_statistics_node(self, file, node_path):
+        """
+        Creates table for image statistics and adds the entries together.
+
+        This does not append rows to the existing table
+        """
+
+        table_in = file.root[node_path]
+
+        if node_path in self.output_file.root:
+            table_out = self.output_file.root[node_path]
+
+            for col in ["counts", "cumulative_counts"]:
+                table_out.modify_column(
+                    colname=col,
+                    column=table_out.col(col) + table_in.col(col),
                 )
+        else:
+            self._copy_node(file, table_in)
 
-        elif "/dl1/service" not in self.output_file:
-            target_group = self.output_file.create_group(
-                "/dl1", "service", createparents=True
-            )
-
-            for node in service_nodes:
-                if node in file:
-                    file.copy_node(node, newparent=target_group)
-
-    def _merge_tel_group(self, file, input_node):
+    def _merge_tel_group(self, file, input_node, filter_columns=None):
         """Add a group that has one child table per telescope (type) to outputfile"""
         if not isinstance(input_node, tables.Group):
             raise TypeError(f"node must be a `tables.Group`, got {input_node}")
@@ -356,38 +361,50 @@ class MergeTool(Tool):
 
         for tel_name, table in input_node._v_children.items():
             if not self.allowed_tels or tel_name in self.allowed_tel_names:
-                self._merge_table(file, table)
+                self._merge_table(file, table, filter_columns=filter_columns)
 
-    def _merge_table(self, file, input_node):
+    def _merge_table(self, file, input_node, filter_columns=None):
         if not isinstance(input_node, tables.Table):
             raise TypeError(f"node must be a `tables.Table`, got {input_node}")
 
         node_path = input_node._v_pathname
         if node_path in self.output_file:
             output_table = self.output_file.get_node(node_path)
-            output_table.append(input_node[:].astype(output_table.dtype))
+            input_table = input_node[:]
+            if filter_columns is not None:
+                input_table = recarray_drop_columns(input_table, filter_columns)
+            output_table.append(input_table.astype(output_table.dtype))
         else:
             group_path, _ = os.path.split(node_path)
             if group_path not in self.output_file:
                 self._create_group(group_path)
 
-            target_group = self.output_file.root[group_path]
-            file.copy_node(node_path, newparent=target_group)
+            if filter_columns is None:
+                self._copy_node(file, input_node)
+            else:
+                input_table = recarray_drop_columns(input_node[:], filter_columns)
+                where, name = os.path.split(node_path)
+                self.output_file.create_table(
+                    where,
+                    name,
+                    filters=input_node.filters,
+                    createparents=True,
+                    obj=input_table,
+                )
 
     def merge_tables(self, file):
         """Go over all mergeable nodes and append to outputfile"""
         for node_path in self.usable_nodes:
-
-            # skip service nodes that should only be included from the first file
-            if node_path in service_nodes:
-                continue
-
             if node_path in file:
                 node = file.root[node_path]
 
                 # nodes with child tables per telescope (type)
                 if node_path in nodes_with_tels:
-                    self._merge_tel_group(file, node)
+                    if node_path == SIMULATED_IMAGE_GROUP and self.skip_images:
+                        filter_columns = ["true_image"]
+                    else:
+                        filter_columns = None
+                    self._merge_tel_group(file, node, filter_columns=filter_columns)
 
                 # groups of tables (e.g. dl2)
                 elif isinstance(node, tables.Group):
@@ -400,7 +417,17 @@ class MergeTool(Tool):
 
     def _create_group(self, node):
         head, tail = os.path.split(node)
-        self.output_file.create_group(head, tail, createparents=True)
+        return self.output_file.create_group(head, tail, createparents=True)
+
+    def _get_or_create_group(self, node):
+        if node in self.output_file.root:
+            return self.output_file.root[node]
+        return self._create_group(node)
+
+    def _copy_node(self, file, node):
+        group_path, _ = os.path.split(node._v_pathname)
+        target_group = self._get_or_create_group(group_path)
+        file.copy_node(node, newparent=target_group)
 
     def start(self):
         merged_files_counter = 0
@@ -428,8 +455,7 @@ class MergeTool(Tool):
                         sys.exit(1)
 
                 self.merge_tables(h5file)
-                if IMAGE_STATISTICS_PATH in h5file:
-                    self.add_image_statistics(h5file)
+                self.add_statistics(h5file)
 
             PROV.add_input_file(str(input_path))
             merged_files_counter += 1
@@ -453,7 +479,7 @@ class MergeTool(Tool):
             product=meta.Product(
                 description="Merged DL1 Data Product",
                 data_category="Sim",  # TODO: copy this from the inputs
-                data_level=datalevels,
+                data_levels=datalevels,
                 data_association="Subarray",
                 data_model_name="ASWG",  # TODO: copy this from inputs
                 data_model_version=self.data_model_version,
