@@ -3,11 +3,13 @@ from astropy import units as u
 from matplotlib import pyplot as plt
 
 from ctapipe.instrument import CameraGeometry
+from ctapipe.utils import get_dataset_path
 from ctapipe.visualization import CameraDisplay
 
 if __name__ == "__main__":
 
-    geom = CameraGeometry.from_name("Whipple109")
+    path = get_dataset_path("Whipple109.camgeom.fits.gz")
+    geom = CameraGeometry.from_table(path)
     image = np.random.uniform(size=geom.pix_id.shape)
 
     plt.figure(figsize=(10, 4))
